﻿using System.Collections;
using System.Collections.ObjectModel;
using AsReaderP252BAndroidDemo.Platforms.Android;
using Com.Asreader.P252b.Device.Type;
using Com.Asreader.P252b.Rfid.Type;

namespace AsReaderP252BAndroidDemo;

public partial class RFIDInventoryPage : ContentPage
{
    private AsReaderP252BMAUIManager asReaderP252BMAUIManager;
    private AsReaderP252BDelegate asReaderP252BDelegate;
    ArrayList powerGainArray;
    int count;
    int powerGain;
    private AsReaderP252BActionState actionState;
    private ObservableCollection<RFIDData> RFIDItems = new ObservableCollection<RFIDData>();
    public ObservableCollection<RFIDData> ItemData { get { return RFIDItems; } }
    private Dictionary<string, RFIDData> dataDic = new Dictionary<string, RFIDData>();


    public RFIDInventoryPage()
	{
		InitializeComponent();
        count = 0;
        powerGain = 5;
        RFIDActivityIndicator.IsRunning = false;
        actionState = AsReaderP252BActionState.Stop;
        asReaderP252BMAUIManager = AsReaderP252BMAUIManager.sharedInstance();
        asReaderP252BDelegate = AsReaderP252BDelegate.sharedInstance();
    }

    protected override void OnAppearing()
    {
        base.OnAppearing();
        StartActivityIndicator();
        RFIDListView.ItemsSource = RFIDItems;
        asReaderP252BDelegate.OnStateChanged = (AsReaderP252BConnectionState state) =>
        {
            OnStateChanged(state);
        };
        asReaderP252BDelegate.OnReadTag = (AsReaderP252BActionState p1, string p2, float p3, float p4, float p5) =>
        {
            OnReadTag(p1, p2, p3, p4, p5);
        };
        asReaderP252BDelegate.OnKeyEvent = (AsReaderP252BKeyState p1) =>
        {
           return OnKeyEvent(p1);
        };
      
        asReaderP252BDelegate.OnActionChanged = (AsReaderP252BActionState p1) =>
        {
            OnActionChanged(p1);
        };
        Task.Run(() =>
        {
            try
            {
                powerGainArray = asReaderP252BMAUIManager.GetPowerGainRange();
                Thread.Sleep(500);
                powerGain = asReaderP252BMAUIManager.PowerGain;
            }
            catch (Exception)
            {
                powerGain = asReaderP252BMAUIManager.GetPowerRangeMax();
            }
            try
            {
                asReaderP252BMAUIManager.SetContinuousMode(true);
            }
            catch (Exception)
            {
            }
            MainThread.BeginInvokeOnMainThread(() =>
            {
                RescanTimePicker.Title = (powerGain / 10).ToString();
                if (powerGainArray != null)
                {
                    foreach (string power in powerGainArray)
                    {
                        RescanTimePicker.Items.Add(power);
                    }
                }
                StopActivityIndicator();
            });
        });
    }
    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        asReaderP252BMAUIManager.Stop();
    }
    void StartScanButton_Clicked(object sender, EventArgs e)
    {
        if (actionState == AsReaderP252BActionState.Inventory)
        {
            asReaderP252BMAUIManager.Stop();
        }
        else if (actionState == AsReaderP252BActionState.Stop)
        {
            asReaderP252BMAUIManager.Inventory();
        }
    }
    void ClearScanButton_Clicked(object sender, EventArgs e)
    {
        count = 0;
        CountLabel.Text = count.ToString();
        dataDic.Clear();
        RFIDItems.Clear();
    }
    void OnPickerSelectedIndexChanged(object sender, EventArgs e)
    {
        var picker = (Picker)sender;
        int selectedIndex = picker.SelectedIndex;
        if (selectedIndex != -1)
        {
            string powerGain = picker.Items[selectedIndex];
            RescanTimePicker.Title = powerGain;
            asReaderP252BMAUIManager.PowerGain = int.Parse(powerGain) * 10;
        }
    }
    private void OnStateChanged(AsReaderP252BConnectionState state)
    {
        if (state != AsReaderP252BConnectionState.Connected)
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                Navigation.PopAsync();
            });
        }
    }
    private void OnReadTag(AsReaderP252BActionState p1, string p2, float p3, float p4, float p5)
    {
        RFIDData InventoryDataChanged;
        if (dataDic.ContainsKey(p2))
        {
            InventoryDataChanged = dataDic[p2];
            InventoryDataChanged.Count++;
            dataDic[p2] = InventoryDataChanged;
        }
        else
        {
            int nowCount = dataDic.Count;
            InventoryDataChanged = new RFIDData(p2, 1);
            dataDic.Add(p2, InventoryDataChanged);
            RFIDItems.Add(InventoryDataChanged);
        }
        count++;
        MainThread.BeginInvokeOnMainThread(() =>
        {
            RFIDListView.ItemsSource = null;
            RFIDListView.ItemsSource = RFIDItems;
            CountLabel.Text = count.ToString();
        });
    }
    private bool OnKeyEvent(AsReaderP252BKeyState p1)
    {
        if (p1 == AsReaderP252BKeyState.KeyDown)
        {
            asReaderP252BMAUIManager.Inventory();
        }
        else
        {
            asReaderP252BMAUIManager.Stop();
        }
        return false;
    }
    private void OnActionChanged(AsReaderP252BActionState p1)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (p1 == AsReaderP252BActionState.Inventory)
            {
                actionState = AsReaderP252BActionState.Inventory;
                StartScanButton.Text = "Stop Scan";
            }
            else if (p1 == AsReaderP252BActionState.Stop)
            {
                actionState = AsReaderP252BActionState.Stop;
                StartScanButton.Text = "Inventory";
            }
        });
    }
    public void StartActivityIndicator()
    {
        RFIDActivityIndicator.IsRunning = true;
        RFIDListView.IsEnabled = false;
        StartScanButton.IsEnabled = false;
        ClearScanButton.IsEnabled = false;
        RescanTimePicker.IsEnabled = false;
    }
    public void StopActivityIndicator()
    {
        RFIDActivityIndicator.IsRunning = false;
        RFIDListView.IsEnabled = true;
        StartScanButton.IsEnabled = true;
        ClearScanButton.IsEnabled = true;
        RescanTimePicker.IsEnabled = true;
    }
}
